import requests


class RequestAPI:
    """Base class for API Wrappers"""

    def __init__(
        self,
        host_url: str,
        api_key: str,
    ):
        """Constructor for connection to Arr API

        Args:
            host_url (str): Host URL to Arr api
            api_key (str): API Key for Arr api
        """
        self.host_url = host_url
        self.api_key = api_key
        self.session = requests.Session()
        self.auth = None

    def basic_auth(self, username, password):
        """If you have basic authentication setup you will need to pass your
        username and passwords to the requests.auth.HTTPBASICAUTH() method.

        Args:
            username (str): Username for basic auth.
            password (str): Password for basic auth.

        Returns:
            Object: HTTP Auth object
        """
        self.auth = requests.auth.HTTPBasicAuth(username, password)
        return self.auth

    def request_get(self, path, params=None):
        """Wrapper on any get requests

        Args:
            path (str): Path to API endpoint e.g. /api/manualimport
            params (dict, optional): URL Parameters to send with the request. Defaults to None.

        Returns:
            Object: Response object from requests
        """
        headers = {"X-Api-Key": self.api_key}
        request_url = "{url}{path}".format(url=self.host_url, path=path)
        res = self.session.get(
            request_url, headers=headers, params=params, auth=self.auth
        )
        return res.json()

    def request_post(self, path, params=None, data=None):
        """Wrapper on any post requests

        Args:
            path (str): Path to API endpoint e.g. /api/manualimport
            params (dict, optional): URL Parameters to send with the request. Defaults to None.
            data (dict, optional): Payload to send with request. Defaults to None.

        Returns:
            Object: Response object from requests
        """
        headers = {"X-Api-Key": self.api_key}
        request_url = "{url}{path}".format(url=self.host_url, path=path)
        res = self.session.post(
            request_url, headers=headers, params=params, json=data, auth=self.auth
        )
        return res.json()

    def request_put(self, path, params=None, data=None):
        """Wrapper on any put requests

        Args:
            path (str): Path to API endpoint e.g. /api/manualimport
            params (dict, optional): URL Parameters to send with the request. Defaults to None.
            data (dict, optional): Payload to send with request. Defaults to None.

        Returns:
            Object: Response object from requests
        """
        headers = {"X-Api-Key": self.api_key}
        request_url = "{url}{path}".format(url=self.host_url, path=path)
        res = self.session.put(
            request_url, headers=headers, params=params, json=data, auth=self.auth
        )
        return res.json()

    def request_del(self, path, params=None, data=None):
        """Wrapper on any delete requests

        Args:
            path (str): Path to API endpoint e.g. /api/manualimport
            params (dict, optional): URL Parameters to send with the request. Defaults to None.
            data (dict, optional): Payload to send with request. Defaults to None.

        Returns:
            Object: Response object from requests
        """
        headers = {"X-Api-Key": self.api_key}
        request_url = "{url}{path}".format(url=self.host_url, path=path)
        res = self.session.delete(
            request_url, headers=headers, params=params, json=data, auth=self.auth
        )
        return res.json()



from datetime import datetime

from .request_api import RequestAPI


class SonarrAPI(RequestAPI):
    """API wrapper for Sonarr endpoints."""

    def _construct_series_json(
        self,
        tvdb_id,
        quality_profile_id,
        root_dir,
        season_folder=True,
        monitored=True,
        ignore_episodes_with_files=False,
        ignore_episodes_without_files=False,
        search_for_missing_episodes=False,
    ):
        """Searches for new shows on trakt and returns Series JSON to add

        Args:
            tvdb_id (int): TVDB id to search
            quality_profile_id (int): Database id for Quality profile
            root_dir (str): Root directory for media
            season_folder (bool, optional): Specify if a season folder should be created. Defaults to True.
            monitored (bool, optional): Specify if the series should be monitored. Defaults to True.
            ignore_episodes_with_files (bool, optional): Ignore episodes that already have files. Defaults to False.
            ignore_episodes_without_files (bool, optional): Ignore episodes that dont have any files. Defaults to False.
            search_for_missing_episodes (bool, optional): Search for any missing episodes and download them. Defaults to False.

        Returns:
            JSON: Array
        """
        series = self.lookup_series_by_tvdb_id(tvdb_id)[0]
        if not monitored and series.get("seasons"):
            for season in series["seasons"]:
                season["monitored"] = False

        series_json = {
            "title": series["title"],
            "seasons": series["seasons"],
            "rootFolderPath": root_dir,
            "qualityProfileId": quality_profile_id,
            "seasonFolder": season_folder,
            "monitored": monitored,
            "tvdbId": tvdb_id,
            "images": series["images"],
            "titleSlug": series["titleSlug"],
            "addOptions": {
                "ignoreEpisodesWithFiles": ignore_episodes_with_files,
                "ignoreEpisodesWithoutFiles": ignore_episodes_without_files,
                "searchForMissingEpisodes": search_for_missing_episodes,
            },
        }
        return series_json

    ## CALENDAR

    # GET /calendar
    def get_calendar(self, start_date=None, end_date=None):
        """Gets upcoming episodes, if start/end are not supplied episodes airing today and tomorrow will be returned

        Args:
            start_date (:obj:`datetime`, optional): ISO8601 start datetime. Defaults to None.
            end_date (:obj:`datetime`, optional): ISO8601 end datetime. Defaults to None.

        Returns:
            JSON: Array
        """
        path = "/api/calendar"
        params = {}
        if start_date:
            params["start"] = datetime.strptime(start_date, "%Y-%m-%d").strftime(
                "%Y-%m-%d"
            )
        if end_date:
            params["end"] = datetime.strptime(end_date, "%Y-%m-%d").strftime("%Y-%m-%d")

        res = self.request_get(path, params=params)
        return res

    ## COMMAND

    # GET /command
    def get_command(self, id_=None):
        """Queries the status of a previously started command, or all currently started commands.

        Args:
            id_ (int, optional): Database id of the command. Defaults to None.

        Returns:
            JSON: Array
        """
        if id_:
            path = f"/api/command/{id_}"
        else:
            path = "/api/command"

        res = self.request_get(path)
        return res

    # POST /command
    def post_command(self, name, **kwargs):
        """Performs any of the predetermined Sonarr command routines

        Note:
            For command names and additional kwargs:
            See https://github.com/Sonarr/Sonarr/wiki/Command

        Args:
            name (str): command name that should be execured
            **kwargs: additional parameters for specific commands



        Returns:
            JSON: Array
        """
        path = "/api/command"
        data = {
            "name": name,
            **kwargs,
        }
        res = self.request_post(path, data=data)
        return res

    ## DISKSPACE

    # GET /diskspace
    def get_disk_space(self):
        """Gets information about Diskspace

        Returns:
            JSON: Array
        """
        path = "/api/diskspace"
        res = self.request_get(path)
        return res

    ## EPISODE

    # GET /episode
    def get_episodes_by_series_id(self, id_):
        """Gets all episodes from a given series ID

        Args:
            id_ (int): Database id for series

        Returns:
            JSON: Array
        """
        path = "/api/episode"
        params = {"seriesId": id_}
        res = self.request_get(path, params=params)
        return res

    # GET /episode/{id}
    def get_episode_by_episode_id(self, id_):
        """Gets a specific episode by database id

        Args:
            id_ (int): Database id for episode

        Returns:
            JSON: Array
        """
        path = f"/api/episode/{id_}"
        res = self.request_get(path)
        return res

    # PUT /episode
    def upd_episode(self, data):
        """Update the given episodes, currently only monitored is changed, all other modifications are ignored.

        Note:
            To be used in conjunction with get_episode()

        Args:
            data (dict): All parameters to update episode

        Returns:
            JSON: Array
        """
        path = "/api/episode"
        res = self.request_put(path, data=data)
        return res

    ## EPISODE FILE

    # GET /episodefile
    def get_episode_files_by_series_id(self, id_):
        """Returns all episode file information for series id specified

        Args:
            id_ (int): Database id of series

        Returns:
            JSON: Array
        """
        path = "/api/episodefile"
        params = {"seriesId": id_}
        res = self.request_get(path, params=params)
        return res

    # GET /episodefile/{id}
    def get_episode_file(self, id_):
        """Returns episode file information for specified id

        Args:
            id_ (int): Database id of episode file

        Returns:
            JSON: Array
        """
        path = f"/api/episodefile/{id_}"
        res = self.request_get(path)
        return res

    # DELETE /episodefile/{id}
    def del_episode_file(self, id_):
        """Deletes the episode file with corresponding id

        Args:
            id_ (int): Database id for episode file

        Returns:
            JSON: {}
        """
        path = f"/api/episodefile/{id_}"
        res = self.request_del(path)
        return res

    # PUT /episodefile/{id}
    def upd_episode_file_quality(self, id_, data):
        """Updates the quality of the episode file and returns the episode file

        Args:
            id_ (int): Database id for episode file
            data (dict): data with quality::

                {
                    "quality": {
                        "quality": {
                        "id": 8
                        },
                        "revision": {
                        "version": 1,
                        "real": 0
                        }
                    },
                }

        Returns:
            JSON: Array
        """
        path = f"/api/episodefile/{id_}"
        res = self.request_put(path, data=data)
        return res

    ## HISTORY

    # GET /history
    def get_history(
        self, sort_key="date", page=1, page_size=10, sort_dir="desc", id_=None
    ):
        """Gets history (grabs/failures/completed)

        Args:
            sort_key (str, optional): series.title or date. Defaults to "date".
            page (int, optional): Page number to return. Defaults to 1.
            page_size (int, optional): Number of items per page. Defaults to 10.
            sort_dir (str, optional): Direction to sort the items. Defaults to "desc".
            id_ (int, optional): Filter to a specific episode ID. Defaults to None.

        Returns:
            JSON: Array
        """
        path = "/api/history"
        params = {
            "sortKey": sort_key,
            "page": page,
            "pageSize": page_size,
            "sortDir": sort_dir,
        }
        if id_:
            params["episodeId"] = id_
        res = self.request_get(path, params=params)
        return res

    ## WANTED (MISSING)

    # GET /wanted/missing
    def get_wanted(self, sort_key="airDateUtc", page=1, page_size=10, sort_dir="asc"):
        """Gets missing episode (episodes without files)

        Args:
            sort_key (str, optional): series.titke or airDateUtc. Defaults to "airDateUtc".
            page (int, optional): Page number to return. Defaults to 1.
            page_size (int, optional): Number of items per page. Defaults to 10.
            sort_dir (str, optional): Direction to sort the items. Defaults to "asc".

        Returns:
            JSON: Array
        """
        path = "/api/wanted/missing"
        params = {
            "sortKey": sort_key,
            "page": page,
            "pageSize": page_size,
            "sortDir": sort_dir,
        }
        res = self.request_get(path, params=params)
        return res

    ## QUEUE

    # GET /queue
    def get_queue(self):
        """Gets currently downloading info

        Returns:
            JSON: Array
        """
        path = "/api/queue"
        res = self.request_get(path)
        return res

    # DELETE /queue
    def del_queue(self, id_, blacklist=False):
        """Deletes an item from the queue and download client. Optionally blacklist item after deletion.

        Args:
            id_ (int): Database id of queue item
            blacklist (bool, optional): Blacklist item after deletion. Defaults to False.

        Returns:
            JSON: {}
        """
        params = {"id": id_, "blacklist": blacklist}
        path = "/api/queue/"
        res = self.request_del(path, params=params)
        return res

    ## PARSE

    # GET /parse
    def get_parsed_title(self, title):
        """Returns the result of parsing a title. series and episodes will be
        returned only if the parsing matches to a specific series and one or more
        episodes. series and episodes will be formatted the same as Series and Episode
        responses.

        Args:
            title (string): Title of series / episode

        Returns:
            JSON: Array
        """
        params = {"title": title}
        path = "/api/parse"
        res = self.request_get(path, params=params)
        return res

    # GET /parse
    def get_parsed_path(self, file_path):
        """Returns the result of parsing a file path. series and episodes will be
        returned only if the parsing matches to a specific series and one or more
        episodes. series and episodes will be formatted the same as Series and Episode
        responses.

        Args:
            file_path (string): file path of series / episode

        Returns:
            JSON: Array
        """
        params = {"path": file_path}
        path = "/api/parse"
        res = self.request_get(path, params=params)
        return res

    ## PROFILE

    # GET /profile
    def get_quality_profiles(self):
        """Gets all quality profiles

        Returns:
            JSON: Array
        """
        path = "/api/profile"
        res = self.request_get(path)
        return res

    ## RELEASE

    # GET /release
    def get_releases(self, id_):
        """Get a release with a specific episode ID.

        Args:
            id_ (int): Database id for episode

        Returns:
            JSON: Array
        """
        params = {"episodeId": id_}
        path = "/api/release"
        res = self.request_get(path, params=params)
        return res

    # POST /release
    def download_release(self, guid, indexer_id):
        """Adds a previously searched release to the download client, if the release is
         still in Sonarr's search cache (30 minute cache). If the release is not found
         in the cache Sonarr will return a 404.

        Args:
            guid (str): Recently searched result guid
            indexer_id (int): Database id of indexer to use

        Returns:
            [type]: [description]
        """
        data = {"guid": guid, "indexerId": indexer_id}
        path = "/api/release"
        res = self.request_post(path, data=data)
        return res

    # POST /release/push
    def push_release(self, title, download_url, protocol, publish_date):
        """If the title is wanted, Sonarr will grab it.

        Args:
            title (str): Release name
            download_url (str): .torrent file URL
            protocol (str): "Usenet" or "Torrent
            publish_date (str): ISO8601 date string

        Returns:
            JSON: Array
        """
        data = {
            "title": title,
            "downloadUrl": download_url,
            "protocol": protocol,
            "publishDate": publish_date,
        }
        path = "/api/release/push"
        res = self.request_post(path, data=data)
        return res

    ## ROOT FOLDER

    # GET /rootfolder
    def get_root_folder(self):
        """Gets root folder

        Returns:
            JSON: Array
        """
        path = "/api/rootfolder"
        res = self.request_get(path)
        return res

    ## SERIES
    # GET /series and /series/{id}
    def get_series(self, id_=None):
        """Returns all series in your collection or the series with the matching
        series ID if one is found.

        Args:
            id_ (int, optional): Database id for series. Defaults to None.

        Returns:
            JSON: Array
        """
        if id_:
            path = f"/api/series/{id_}"
        else:
            path = "/api/series"

        res = self.request_get(path)
        return res

    # POST /series
    def add_series(
        self,
        tvdb_id,
        quality_profile_id,
        root_dir,
        season_folder=True,
        monitored=True,
        ignore_episodes_with_files=False,
        ignore_episodes_without_files=False,
        search_for_missing_episodes=False,
    ):
        """Adds a new series to your collection

        Note:
            if you do not add the required params, then the series wont function. some of these without the others can
            indeed make a "series". But it wont function properly in nzbdrone.

        Args:
            tvdb_id (int): TDDB Id
            quality_profile_id (int): Database id for quality profile
            root_dir (str): Root folder location, full path will be created from this
            season_folder (bool, optional): Create a folder for each season. Defaults to True.
            monitored (bool, optional): Monitor this series. Defaults to True.
            ignore_episodes_with_files (bool, optional): Ignore any episodes with existing files. Defaults to False.
            ignore_episodes_without_files (bool, optional): Ignore any episodes without existing files. Defaults to False.
            search_for_missing_episodes (bool, optional): Search for missing episodes to download. Defaults to False.

        Returns:
            JSON: Array
        """
        series_json = self._construct_series_json(
            tvdb_id,
            quality_profile_id,
            root_dir,
            season_folder,
            monitored,
            ignore_episodes_with_files,
            ignore_episodes_without_files,
            search_for_missing_episodes,
        )

        path = "/api/series"
        res = self.request_post(path, data=series_json)
        return res

    # PUT /series
    def upd_series(self, data):
        """Update an existing series

        Args:
            data (dict): contains data obtained by get_series()

        Returns:
            JSON: Array
        """
        path = "/api/series"
        res = self.request_put(path, data=data)
        return res

    # DELETE /series/{id}
    def del_series(self, id_, delete_files=False):
        """Delete the series with the given id

        Args:
            id_ (int): Database id for series
            delete_files (bool, optional): If true series folder and files will be deleted. Defaults to False.

        Returns:
            JSON: {}
        """
        # File deletion does not work
        params = {"deleteFiles": delete_files}
        path = f"/api/series/{id_}"
        res = self.request_del(path, params=params)
        return res

    # GET /series/lookup
    def lookup_series(self, term):
        """Searches for new shows on TheTVDB.com utilizing sonarr.tv's caching and augmentation proxy.

        Args:
            term (str): Series' Name, using `%20` to signify spaces, as in `The%20Blacklist`

        Returns:
            JSON: Array
        """
        params = {"term": term}
        path = "/api/series/lookup"
        res = self.request_get(path, params=params)
        return res

    # GET /series/lookup
    def lookup_series_by_tvdb_id(self, id_):
        """Searches for new shows on TheTVDB.com utilizing sonarr.tv's caching and augmentation proxy.

        Args:
            id_ (int): TVDB ID

        Returns:
            JSON: Array
        """
        params = {"term": f"tvdb:{id_}"}
        path = "/api/series/lookup"
        res = self.request_get(path, params=params)
        return res

    ## SYSTEM

    # GET /system/status
    def get_system_status(self):
        """Returns system status

        Returns:
            JSON: Array
        """
        path = "/api/system/status"
        res = self.request_get(path)
        return res

    # GET /system/backup
    def get_backup(self):
        """Returns the list of available backups

        Returns:
            JSON: Array
        """
        path = "/api/system/backup"
        res = self.request_get(path)
        return res

    ## TAG

    # GET /tag and /tag/{id}
    def get_tag(self, id_=None):
        """Returns all tags or specific tag by database id

        Args:
            id_ (int, optional): Database id for tag. Defaults to None.

        Returns:
            JSON: Array
        """
        if not id_:
            path = "/api/tag"
        else:
            path = f"/api/tag/{id_}"

        res = self.request_get(path)
        return res

    # POST /tag
    def create_tag(self, label):
        """Adds a new tag

        Args:
            label (str): Tag name / label

        Returns:
            JSON: Array
        """
        data = {"label": label}
        path = "/api/tag"
        res = self.request_post(path, data=data)
        return res

    # PUT /tag/{id}
    def upd_tag(self, id_, label):
        """Update an existing tag

        Note:
            You should perform a get_tag() and submit the full body with changes

        Args:
            id_ (int): Database id of tag
            label (str): tag name / label

        Returns:
            JSON: Array
        """
        data = {"id": id_, "label": label}
        path = f"/api/tag/{id_}"
        res = self.request_put(path, data=data)
        return res

    # DELETE /tag/{id}
    def del_tag(self, id_):
        """Delete the tag with the given ID

        Args:
            id_ (int): Database id of tag

        Returns:
            JSON: {}
        """
        path = f"/api/tag/{id_}"
        res = self.request_del(path)
        return res

    ## LOG

    # GET /log
    def get_logs(
        self,
        page=1,
        page_size=10,
        sort_key="time",
        sort_dir="desc",
        filter_key=None,
        filter_value="All",
    ):
        """Gets sonarr logs

        Args:
            page (int, optional): Specifiy page to return. Defaults to 1.
            page_size (int, optional): Number of items per page. Defaults to 10.
            sort_key (str, optional): Field to sort by. Defaults to "time".
            sort_dir (str, optional): Direction to sort. Defaults to "desc".
            filter_key (str, optional): Key to filter by. Defaults to None.
            filter_value (str, optional): Value of the filter. Defaults to "All".

        Returns:
            JSON: Array
        """
        path = "/api/log"
        params = {
            "page": page,
            "pageSize": page_size,
            "sortKey": sort_key,
            "sortDir": sort_dir,
            "filterKey": filter_key,
            "filterValue": filter_value,
        }
        res = self.request_get(path, params=params)
        return res



from .request_api import RequestAPI


class RadarrAPI(RequestAPI):
    """API wrapper for Radarr endpoints.

    Args:
        RequestAPI (:obj:`str`): provides connection to API endpoint
    """

    def _construct_movie_json(
        self,
        db_id,
        quality_profile_id,
        root_dir,
        monitored=True,
        search_for_movie=True,
        tmdb=True,
    ):
        """Searches for movie on tmdb and returns Movie json to add.

        Args:
            db_id (str): imdb or tmdb id
            quality_profile_id (int): ID of the quality profile the movie will use
            root_dir (str): location of the root DIR
            monitored (bool, optional): should the movie be monitored. Defaults to True.
            search_for_movie (bool, optional): Should we search for the movie. Defaults to True.
            tmdb (bool, optional): Use TMDB IDs. Set to False to use IMDB. Defaults to True.

        Raises:
            Exception: [description]

        Returns:
            JSON: Movie JSON for adding a movie to radarr
        """
        if tmdb:
            movie = self.lookup_movie_by_tmdb_id(db_id)[0]
        else:
            movie = self.lookup_movie_by_imdb_id(db_id)[0]

        if not movie:
            raise Exception("Movie Doesn't Exist")

        movie_json = {
            "title": movie["title"],
            "rootFolderPath": root_dir,
            "qualityProfileId": quality_profile_id,
            "year": movie["year"],
            "tmdbId": movie["tmdbId"],
            "images": movie["images"],
            "titleSlug": movie["titleSlug"],
            "monitored": monitored,
            "addOptions": {"searchForMovie": search_for_movie},
        }
        return movie_json

    ## MOVIE

    # GET /movie
    def get_movie(self, id_=None):
        """Returns all movies in the database, or returns a movie with a specific TMDB ID.

        Args:
            id_ (int, optional): TMDB Id of Movies. Defaults to None.

        Returns:
            JSON: List of Movies from Radarr database
        """
        params = {}
        if id_:
            params["tmdbId"] = id_
        path = "/api/v3/movie"
        res = self.request_get(path, params=params)
        return res

    # POST /movie
    def add_movie(
        self,
        db_id,
        quality_profile_id,
        root_dir,
        monitored=True,
        search_for_movie=True,
        tmdb=True,
    ):
        """Adds a movie to the database

        Args:
            db_id (str): IMDB or TMDB ID
            quality_profile_id (int): ID of the quality profile the movie will use
            root_dir (str): location of the root DIR
            monitored (bool, optional): should the movie be monitored. Defaults to True.
            search_for_movie (bool, optional): Should we search for the movie. Defaults to True.
            tmdb (bool, optional): Use TMDB IDs. Set to False to use IMDB. Defaults to True.

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        movie_json = self._construct_movie_json(
            db_id, quality_profile_id, root_dir, monitored, search_for_movie, tmdb
        )

        path = "/api/v3/movie"
        res = self.request_post(path, data=movie_json)
        return res

    # PUT /movie
    def upd_movie(self, data, move_files=False):
        """Updates a movie in the database.

        Args:
            data (dict): Dictionary containing an object obtained from get_movie()
            move_files (bool, optional): Have radarr move files when updating. Defaults to False.

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """

        path = "/api/movie"
        params = {"moveFiles": move_files}
        res = self.request_put(path, data=data, params=params)
        return res

    # GET /movie/{id}
    def get_movie_by_movie_id(self, id_):
        """Get a movie by the Radarr database ID

        Args:
            id_ (int): Database Id of movie to return

        Returns:
            JSON: Movie data if present in database
        """
        path = f"/api/v3/movie/{id_}"
        res = self.request_get(path)
        return res

    # DELETE /movie/{id}
    def del_movie(self, id_, delete_files=False, add_exclusion=False):
        """Delete a single movie by database id.

        Args:
            id_ (int): Database Id of movie to delete.
            delete_files (bool, optional): Delete movie files when deleting movies. Defaults to False.
            add_exclusion (bool, optional): Add deleted movies to List Exclusions. Defaults to False.

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        params = {"deleteFiles": delete_files, "addExclusion": add_exclusion}
        path = f"/api/v3/movie/{id_}"
        res = self.request_del(path, params=params)
        return res

    # GET /movie/lookup
    def lookup_movie(self, term):
        """Search for a movie to add to the database (Uses TMDB for search results)

        Args:
            term (str): search term to use for lookup

        Returns:
            JSON: List of movies found
        """
        params = {"term": term}
        path = "/api/v3/movie/lookup"
        res = self.request_get(path, params=params)
        return res

    # GET /movie/lookup
    def lookup_movie_by_tmdb_id(self, id_):
        """Search for movie by TMDB ID

        Args:
            id_ (str): TMDB ID

        Returns:
            JSON: List of movies found
        """
        params = {"term": f"tmdb:{id_}"}
        path = "/api/v3/movie/lookup"
        res = self.request_get(path, params=params)
        return res

    # GET /movie/lookup
    def lookup_movie_by_imdb_id(self, id_):
        """Search for movie by IMDB ID

        Args:
            id_ (str): IMDB ID

        Returns:
            JSON: List of movies found
        """
        params = {"term": f"imdb:{id_}"}
        path = "/api/v3/movie/lookup"
        res = self.request_get(path, params=params)
        return res

    # PUT /movie/editor
    def upd_movies(self, data):
        """The Updates operation allows to edit properties of multiple movies at once

        Args:
            data (dict): Updated movie information

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = "/api/v3/movie/editor"
        res = self.request_put(path, data=data)
        return res

    # DELETE /movie/editor
    def del_movies(self, data):
        """The delete operation allows mass deletion of movies (and optionally files)

        Args:
            data (dict): dictionary of movies to be deleted::

                {
                    "movieIds": [
                        0
                    ],
                    "deleteFIles": true,
                    "addImportExclusion": true
                }

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = "/api/v3/movie/editor"
        res = self.request_del(path, data=data)
        return res

    # POST /movie/import
    def import_movies(self, data):
        """The movie import endpoint is used by the bulk import view in Radarr UI. It allows movies to be bulk added to the Radarr database.

        Args:
            data (dict): dictionary of all movies to be imported

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = "/api/v3/movie/import"
        res = self.request_post(path, data=data)
        return res

    ## MOVIEFILE

    # GET /moviefile
    def get_movie_files_by_movie_id(self, id_):
        """Get a movie file object by Movie database id.

        Args:
            id_ (int): Movie database id

        Returns:
            JSON: Movie file information if exists
        """
        params = {"movieid": id_}
        path = "/api/v3/moviefile"
        res = self.request_get(path, params=params)
        return res

    # GET /moviefile
    def get_movie_files(self, moviefile_ids):
        """Get movie file information for multiple movie files

        Args:
            moviefile_ids (array): an array of movie file ids

        Returns:
            JSON: List of movie files
        """
        params = {"moviefileids": moviefile_ids}
        path = "/api/v3/moviefile"
        res = self.request_get(path, params=params)
        return res

    # GET /moviefile/{id}
    def get_movie_file(self, id_):
        """Get movie file by database id

        Args:
            id_ (int): movie file id

        Returns:
            JSON: movie file information
        """
        path = f"/api/v3/moviefile/{id_}"
        res = self.request_get(path)
        return res

    # DELETE /moviefile/{id}
    def del_movie_file(self, id_):
        """Allows for deletion of a moviefile by its database id.

        Args:
            id_ (int): Movie file id

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = f"/api/v3/movie/{id_}"
        res = self.request_del(path)
        return res

    ## HISTORY

    # GET /history
    def get_history(
        self, page=1, page_size=20, sort_direction="descending", sort_key="date"
    ):
        """Return a json object list of items in your history

        Args:
            page (int, optional): Page to be returned. Defaults to 1.
            page_size (int, optional): Number of results per page. Defaults to 20.
            sort_direction (str, optional): Direction to sort items. Defaults to "descending".
            sort_key (str, optional): Field to sort by. Defaults to "date".

        Returns:
            JSON: Array
        """
        params = {
            "page": page,
            "pageSize": page_size,
            "sortDirection": sort_direction,
            "sortKey": sort_key,
        }
        path = "/api/v3/history"
        res = self.request_get(path, params=params)
        return res

    # GET /history/movie
    def get_movie_history(self, id_, event_type=None):
        """Get history for a given movie in database by its database id

        Args:
            id_ (int): Database id of movie
            event_type (int, optional): History event type to retrieve. Defaults to None.

        Returns:
            JSON: Array
        """
        params = {"movieId": id_}
        if event_type:
            params["eventType"] = event_type
        path = "/api/v3/history/movie"
        res = self.request_get(path, params=params)
        return res

    ## BLACKLIST

    # GET /blacklist
    def get_blacklist(
        self,
        page=1,
        page_size=20,
        sort_direction="descending",
        sort_key="date",
    ):
        """Returns blacklisted releases.

        Args:
            page (int, optional): Page to be returned. Defaults to 1.
            page_size (int, optional): Number of results per page. Defaults to 20.
            sort_direction (str, optional): Direction to sort items. Defaults to "descending".
            sort_key (str, optional): Field to sort by. Defaults to "date".

        Returns:
            JSON: Array
        """
        params = {
            "page": page,
            "pageSize": page_size,
            "sortDirection": sort_direction,
            "sortKey": sort_key,
        }
        path = "/api/v3/blacklist"
        res = self.request_get(path, params=params)
        return res

    # DELETE /blacklist
    def del_blacklist(self, id_):
        """Removes a specific release (the id provided) from the blacklist

        Args:
            id_ (int): blacklist id from database

        Returns:
            JSON: Array
        """
        params = {"id": id_}
        path = "/api/v3/blacklist"
        res = self.request_del(path, params=params)
        return res

    # GET /blacklist/movie
    def get_blacklist_by_movie_id(
        self,
        id_,
    ):
        """Retrieves blacklisted releases that are tied to a given movie in the database.

        Args:
            id_ (int): Movie id from Database

        Returns:
            JSON: Array
        """
        params = {"movieId": id_}
        path = "/api/v3/blacklist/movie"
        res = self.request_get(path, params=params)
        return res

    # DELETE /blacklist/bulk
    def del_blacklist_bulk(self, data):
        """Delete blacklisted releases in bulk

        Args:
            data (dict): blacklists that should be deleted

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = "/api/v3/blacklist/bulk"
        res = self.request_del(path, data=data)
        return res

    ## QUEUE

    # GET /queue
    def get_queue(
        self,
        page=1,
        page_size=20,
        sort_direction="ascending",
        sort_key="timeLeft",
        include_unknown_movie_items=True,
    ):
        """Return a json object list of items in the queue

        Args:
            page (int, optional): Page to be returned. Defaults to 1.
            page_size (int, optional): Number of results per page. Defaults to 20.
            sort_direction (str, optional): Direction to sort items. Defaults to "ascending".
            sort_key (str, optional): Field to sort by. Defaults to "timeLeft".
            include_unknown_movie_items (bool, optional): Include unknown movie items. Defaults to True.

        Returns:
            JSON: Array
        """
        params = {
            "page": page,
            "pageSize": page_size,
            "sortDirection": sort_direction,
            "sortKey": sort_key,
            "includeUnknownMovieItems": include_unknown_movie_items,
        }
        path = "/api/v3/queue"
        res = self.request_get(path, params=params)
        return res

    # DELETE /queue/{id}
    def del_queue(self, id_, remove_from_client=True, blacklist=True):
        """Remove an item from the queue and optionally blacklist it

        Args:
            id_ (int): id of the item to be removed
            remove_from_client (bool, optional): Remove the item from the client. Defaults to True.
            blacklist (bool, optional): Add the item to the blacklist. Defaults to True.

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        params = {"removeFromClient": remove_from_client, "blacklist": blacklist}
        path = f"/api/v3/queue/{id_}"
        res = self.request_del(path, params=params)
        return res

    # DELETE /queue/bulk
    def del_queue_bulk(self, data, remove_from_client=True, blacklist=True):
        """Remove multiple items from queue by their ids

        Args:
            data (dict): Dictionary of IDs to be removed::

                {
                "ids": [
                    0
                ]
                }

            remove_from_client (bool, optional): Remove the items from the client. Defaults to True.
            blacklist (bool, optional): Add the items to the blacklist. Defaults to True.

        Returns:
            JSON: 200 ok, 401 invalid api key
        """
        params = {"removeFromClient": remove_from_client, "blacklist": blacklist}
        path = "/api/v3/queue/bulk"
        res = self.request_del(path, params=params, data=data)
        return res

    # GET /queue/details
    def get_queue_details(
        self,
        include_movie=True,
    ):
        """Get details of all items in queue

        Args:
            include_movie (bool, optional): Include movie object if linked. Defaults to True.

        Returns:
            JSON: Array
        """
        params = {
            "includeMovie": include_movie,
        }
        path = "/api/v3/queue/details"
        res = self.request_get(path, params=params)
        return res

    # GET /queue/status
    def get_queue_status(self):
        """Stats on items in queue

        Returns:
            JSON: Array
        """
        path = "/api/v3/queue/status"
        res = self.request_get(path)
        return res

    # POST /queue/grab/{id}
    def force_grab_queue_item(self, id_):
        """Perform a Radarr "force grab" on a pending queue item by its ID.

        Args:
            id_ (int): queue item id from database.

        Returns:
            JSON: 200 ok, 401 Invalid API Key
        """
        path = f"/api/v3/queue/grab/{id_}"
        res = self.request_post(path)
        return res

    ## INDEXER

    # GET /indexer and /indexer/{id}
    def get_indexer(self, id_=None):
        """Get all indexers or a single indexer by its database id.

        Args:
            id_ (int, optional): indexer database id. Defaults to None.

        Returns:
            JSON: Array
        """
        if not id_:
            path = "/api/v3/indexer"
        else:
            path = f"/api/v3/indexer/{id_}"

        res = self.request_get(path)
        return res

    # PUT /indexer/{id}
    def upd_indexer(self, id_, data):
        """Edit an indexer

        Args:
            id_ (int): Database id of indexer
            data (dict): information to be changed within the indexer

        Returns:
            JSON: 200 ok, 401 Unauthorized
        """
        path = f"/api/v3/indexer/{id_}"
        res = path.request_put(path, data=data)
        return res

    # DELETE /indexer/{id}
    def del_indexer(self, id_):
        """Delete indexer by database id

        Args:
            id_ (int): DAtabase id of the indexer

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = f"/api/v3/indexer/{id_}"
        res = self.request_del(path)
        return res

    ## DOWNLOAD CLIENT

    # GET /downloadclient and /downloadclient/{id}
    def get_download_client(self, id_=None):
        """Get a list of all the download clients or a single client by its database id added in Radarr

        Args:
            id_ (int, optional): Download client database id. Defaults to None.

        Returns:
            JSON: Array
        """
        if not id_:
            path = "/api/v3/downloadclient"
        else:
            path = f"/api/v3/downloadclient/{id_}"

        res = self.request_get(path)
        return res

    # PUT /downloadclient/{id}
    def upd_download_client(self, id_, data):
        """Edit a downloadclient by database id

        Args:
            id_ (int): Download client database id
            data (dict): data to be updated within download client

        Returns:
            JSON: 200 Ok
        """
        path = f"/api/v3/downloadclient/{id_}"
        res = path.request_put(path, data=data)
        return res

    # DELETE /downloadclient/{id}
    def del_download_client(self, id_):
        """Delete a download client by database id

        Args:
            id_ (int): download client database id

        Returns:
            JSON: 200 Ok
        """
        path = f"/api/v3/downloadclient/{id_}"
        res = self.request_del(path)
        return res

    ## IMPORT LISTS

    # GET /importlist and /importlist/{id}
    def get_import_list(self, id_=None):
        """Query Radarr for all lists or a single list by its database id

        Args:
            id_ (int, optional): Import list database id. Defaults to None.

        Returns:
            JSON: Array
        """
        if not id_:
            path = "/api/v3/importlist"
        else:
            path = f"/api/v3/importlist/{id_}"

        res = self.request_get(path)
        return res

    # PUT /importlist/{id}
    def upd_import_list(self, id_, data):
        """Edit an importlist

        Args:
            id_ (int): Import list database id
            data (dict): data to be updated within the import list

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = f"/api/v3/importlist/{id_}"
        res = path.request_put(path, data=data)
        return res

    # DELETE /importlist/{id}
    def del_import_list(self, id_):
        """Delete an import list

        Args:
            id_ (int): Import list database id

        Returns:
            JSON: 200 ok, 401 Unauthorized
        """
        path = f"/api/v3/importlist/{id_}"
        res = self.request_del(path)
        return res

    ## NOTIFICATION

    # GET /notification and /notification/{id}
    def get_notification(self, id_=None):
        """Get all notifications or a single notification by its database id

        Args:
            id_ (int, optional): Notification database id. Defaults to None.

        Returns:
            JSON: Array
        """
        if not id_:
            path = "/api/v3/notification"
        else:
            path = f"/api/v3/notification/{id_}"

        res = self.request_get(path)
        return res

    # PUT /notification/{id}
    def upd_notification(self, id_, data):
        """Edit notification by database id

        Args:
            id_ (int): Database id of notification
            data (dict): data that requires updating

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = f"/api/v3/notification/{id_}"
        res = path.request_put(path, data=data)
        return res

    # DELETE /notification/{id}
    def del_notification(self, id_):
        """Delete a notification by its database id

        Args:
            id_ (int): Database id of notification

        Returns:
            JSON: 201 Ok, 401 Unauthorized
        """
        path = f"/api/v3/notification/{id_}"
        res = self.request_del(path)
        return res

    ## TAG

    # GET /tag and /tag/{id}
    def get_tag(self, id_=None):
        """Get all tags in the database or return a tag by database id.

        Args:
            id_ (int, optional): Database if of tag. Defaults to None.

        Returns:
            JSON: Array
        """
        if not id_:
            path = "/api/v3/tag"
        else:
            path = f"/api/v3/tag/{id_}"

        res = self.request_get(path)
        return res

    # POST /tag
    def create_tag(self, label):
        """Create a new tag that can be assigned to a movie, list, delay profile, notification, or restriction

        Args:
            label (str): Label of the tag.

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        data = {"id": 0, "label": label}
        path = "/api/v3/tag"
        res = self.request_post(path, data=data)
        return res

    # PUT /tag/{id}
    def upd_tag(self, id_, label):
        """Edit a tag by its database id

        Args:
            id_ (int): Database id of tag
            label (str): new label of the tag

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        data = {"id": id_, "label": label}
        path = f"/api/v3/tag/{id_}"
        res = self.request_put(path, data=data)
        return res

    # DELETE /tag/{id}
    def del_tag(self, id_):
        """Delete a tag

        Args:
            id_ (int): Database id of tag

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = f"/api/v3/tag/{id_}"
        res = self.request_del(path)
        return res

    # GET /tag/detail and /tag/detail/{id}
    def get_tag_details(self, id_=None):
        """Get a list of tag detail objects for all tags in the database,
        or the id of all items in the database which use the specified tag ID.

        Args:
            id_ (int, optional): Database id of tag. Defaults to None.

        Returns:
            JSON: Array
        """
        if not id_:
            path = "/api/v3/tag/detail"
        else:
            path = f"/api/v3/tag/detail/{id_}"

        res = self.request_get(path)
        return res

    ## DISKSPACE

    # GET /diskspace
    def get_disk_space(self):
        """Query Radarr for disk usage information
            System > Status

        Returns:
            JSON: Array
        """
        path = "/api/v3/diskspace"
        res = self.request_get(path)
        return res

    ## SETTINGS

    # GET /config/ui
    def get_config_ui(self):
        """Query Radarr for UI settings

        Returns:
            JSON: Array
        """
        path = "/api/v3/config/ui"
        res = self.request_get(path)
        return res

    # PUT /config/ui
    def upd_config_ui(self, data):
        """Edit one or many UI settings and save to to the database

        Args:
            data (dict): data to be Updated

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = "/api/v3/config/ui"
        res = self.request_put(path, data=data)
        return res

    # GET /config/host
    def get_config_host(self):
        """Get General/Host settings for Radarr.

        Returns:
            JSON: Array
        """
        path = "/api/v3/config/host"
        res = self.request_get(path)
        return res

    # PUT /config/host
    def upd_config_host(self, data):
        """Edit General/Host settings for Radarr.

        Args:
            data (dict): data to bu updated

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = "/api/v3/config/host"
        res = self.request_put(path, data=data)
        return res

    # GET /config/naming
    def get_config_naming(self):
        """Get Settings for movie file and folder naming.

        Returns:
            JSON: Array
        """
        path = "/api/v3/config/naming"
        res = self.request_get(path)
        return res

    # PUT /config/naming
    def upd_config_naming(self, data):
        """Edit Settings for movie file and folder naming.

        Args:
            data (dict): data to be updated

        Returns:
            JSON: 200 Ok, 401 Unauthorized
        """
        path = "/api/v3/config/naming"
        res = self.request_put(path, data=data)
        return res

    ## METADATA

    # GET /metadata
    def get_metadata(self):
        """Get all metadata consumer settings

        Returns:
            JSON: Array
        """
        path = "/api/v3/metadata"
        res = self.request_get(path)
        return res

    ## SYSTEM

    # GET /system/status
    def get_system_status(self):
        """Find out information such as OS, version, paths used, etc

        Returns:
            JSON: Array
        """
        path = "/api/v3/system/status"
        res = self.request_get(path)
        return res

    ## HEALTH

    # GET /health
    def get_health(self):
        """Query radarr for health information

        Returns:
            JSON: Array
        """
        path = "/api/v3/health"
        res = self.request_get(path)
        return res

    ## COMMAND

    # POST /command
    def post_command(self, name, **kwargs):
        """Performs any of the predetermined Radarr command routines.

        Note:
            For command names and kwargs:
            See https://radarr.video/docs/api/#/Command/post-command

        Args:
            name (str): Name of the command to be run
            **kwargs: additional parameters for specific commands

        Returns:
            JSON: Array
        """
        data = {
            "name": name,
            **kwargs,
        }
        path = "/api/v3/command"
        res = self.request_post(path, data=data)
        return res

    ## UPDATE

    # GET /update
    def get_updates(self):
        """Will return a list of recent updated to Radarr

        Returns:
            JSON: Array
        """
        path = "/api/v3/update"
        res = self.request_get(path)
        return res

    ## QUALITY PROFILE

    # GET /qualityProfile
    def get_quality_profiles(self):
        """Query Radarr for quality profiles

        Returns:
            JSON: Array
        """
        path = "/api/v3/qualityProfile"
        res = self.request_get(path)
        return res

    ## CALENDAR

    # GET /calendar
    def get_calendar(self, start_date, end_date, unmonitored=True):
        """Get a list of movies based on calendar Parameters

        Args:
            start_date (:obj:`datetime`): ISO 8601 start datetime
            end_date (:obj:`datetime`): ISO 8601 end datetime
            unmonitored (bool, optional): Include unmonitored movies. Defaults to True.

        Returns:
            JSON: Array
        """
        params = {
            "start": start_date.strftime("%Y-%m-%d"),
            "end": end_date.strftime("%Y-%m-%d"),
            "unmonitored": unmonitored,
        }
        path = "/api/v3/calendar"
        res = self.request_get(path, params=params)
        return res

    ## CUSTOM FILTERS

    # GET /customfilter
    def get_custom_filter(self):
        """Query Radarr for custom filters

        Returns:
            JSON: Array
        """
        path = "/api/v3/customfilter"
        res = self.request_get(path)
        return res

    ## REMOTE PATH MAPPING

    # GET /remotePathMapping
    def get_remote_path_mapping(self):
        """Get a list of remote paths being mapped and used by Radarr

        Returns:
            JSON: Array
        """
        path = "/api/v3/remotePathMapping"
        res = self.request_get(path)
        return res

    ## ROOT FOLDER

    # GET /rootfolder
    def get_root_folder(self):
        """Query Radarr for root folder information

        Returns:
            JSON: Array
        """
        path = "/api/v3/rootfolder"
        res = self.request_get(path)
        return res



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyarr']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.21.0,<3.0.0']

setup_kwargs = {
    'name': 'pyarr',
    'version': '2.0.6',
    'description': 'A Sonarr and Radarr API Wrapper',
    'long_description': "# Sonarr and Radarr API Python Wrapper\n\nPython Wrapper for the [Sonarr](https://github.com/Sonarr/Sonarr) and [Radarr](https://github.com/Radarr/Radarr) API.\n\nSee the full [documentation](https://docs.totaldebug.uk/pyarr/) for supported functions.\n\n### Requirements\n\n-   requests\n\n### Example Sonarr Usage:\n\n```python\n# Import SonarrAPI Class\nfrom pyarr import SonarrAPI\n\n# Set Host URL and API-Key\nhost_url = 'http://your-domain.com'\n\n# You can find your API key in Settings > General.\napi_key = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'\n\n# Instantiate SonarrAPI Object\nsonarr = SonarrAPI(host_url, api_key)\n\n# Get and print TV Shows\nprint(sonarr.get_series(123))\n```\n\n### Example Radarr Usage:\n\n```python\n# Import RadarrAPI Class\nfrom pyarr import RadarrAPI\n\n# Set Host URL and API-Key\nhost_url = 'http://your-domain.com'\n\n# You can find your API key in Settings > General.\napi_key = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'\n\n# Instantiate RadarrAPI Object\nradarr = RadarrAPI(host_url, api_key)\n\n# Get and print TV Shows\nprint(radarr.get_root_folder())\n```\n\n### Documentation\n\n-   [Pyarr Documentation](https://docs.totaldebug.uk/pyarr)\n-   [Sonarr API Documentation](https://github.com/Sonarr/Sonarr/wiki/API)\n-   [Radarr API Documentation](https://radarr.video/docs/api)\n",
    'author': 'Steven Marks',
    'author_email': 'marksie1988@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/totaldebug/pyarr',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
